@@ -1,5 +1,9 @@
-## Copyright (C) 2016-2019 Kai T. Ohlhus <k.ohlhus@gmail.com>
-## Copyright (C) 2010 Fotios Kasolis <fotios.kasolis@gmail.com>
+########################################################################
+##
+## Copyright (C) 2010-2022 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -16,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} publish (@var{file})
@@ -27,10 +33,12 @@
 ## Generate a report from the Octave script file @var{file} in one of several
 ## output formats.
 ##
-## The generated reports interpret any Publishing Markup in comments, which is
-## explained in detail in the GNU Octave manual.  Assume the following example,
-## using some Publishing Markup, to be the contents of the script file
-## @file{pub_example.m}:
+## The generated reports interpret Publishing Markup in section comments, which
+## is explained in detail in the GNU Octave manual.  Section comments are
+## comment blocks that start with a line with double comment character.
+##
+## Assume the following example, using some Publishing Markup, to be the
+## contents of the script file @file{pub_example.m}:
 ##
 ## @example
 ## @group
@@ -55,12 +63,11 @@
 ##
 ## To publish this script file, type @code{publish ("pub_example.m")}.
 ##
-## With only @var{file} given, a HTML report is generated in a subdirectory
-## @file{html} relative to the current working directory.  The Octave commands
-## are evaluated in a separate context and any figures created while executing
-## the script file are included in the report.  All formatting syntax of
-## @var{file} is treated according to the specified output format and included
-## in the report.
+## When called with one input argument, a HTML report is generated in a
+## subdirectory @file{html} relative to the current working directory.  Any
+## Octave commands in @file{pub_example.m} are evaluated in a separate context
+## and any figures created while executing the script file are included in the
+## report.
 ##
 ## Using @code{publish (@var{file}, @var{output_format})} is equivalent to the
 ## function call using a structure
@@ -203,7 +210,7 @@ function output_file = publish (file, varargin)
   ## Check file to be in Octave's load path
   [file_path, file_name, file_ext] = fileparts (file);
   if (isempty (file_path))
-    file_path = pwd;
+    file_path = pwd ();
   endif
   if (exist ([file_name, file_ext]) != 2)
     error (["publish: " file " is not in the load path"]);
@@ -402,6 +409,7 @@ endfunction
 
 
 function doc = parse_m_source (doc)
+
   ## PARSE_M_SOURCE First parsing level
   ##   This function extracts the overall structure (paragraphs and code
   ##   sections) given in doc.m_source.
@@ -427,6 +435,7 @@ function doc = parse_m_source (doc)
   ## Checks line to have N "%" or "#" lines
   ## followed either by a space or end of string
   function r = is_publish_markup (cstr, N)
+
     str = char (cstr);
 
     r = any (strncmp (str, {"%%%", "##"}, N));
@@ -564,6 +573,7 @@ endfunction
 
 
 function p_content = parse_paragraph_content (content)
+
   ## PARSE_PARAGRAPH_CONTENT second parsing level
   ##
   ##   Parses the content of a paragraph (without potential title) and
@@ -605,7 +615,7 @@ function p_content = parse_paragraph_content (content)
   ## Extract <html> and <latex> blocks recursively.
   content_str = strjoin (content, "\n");
   tags = {"html", "latex"};
-  for i = 1:length(tags)
+  for i = 1:length (tags)
     tok = regexp (content_str, ...
       ['(.*?)(^|\n\n)(<', tags{i}, '>)\n(.*?)\n(<\/', ...
         tags{i}, '>)($|\n\n)(.*)'], "tokens", "once");
@@ -703,12 +713,13 @@ function p_content = parse_paragraph_content (content)
     p_content{end+1}.type = "text";
     p_content{end}.content = strjoin (block, "\n");
   endfor
+
 endfunction
 
 
 function m_source = read_file_to_cellstr (file)
-  ## READ_FILE_TO_CELLSTR reads a given file line by line into a cellstring
 
+  ## READ_FILE_TO_CELLSTR reads a given file line by line into a cellstring
   fid = fopen (file, "r");
   i = 0;
   do
@@ -716,12 +727,13 @@ function m_source = read_file_to_cellstr (file)
   until (! ischar (m_source{i}))
   fclose (fid);
   m_source = m_source(1:end-1);  # No EOL
+
 endfunction
 
 
 function ofile = create_output (doc, options)
-  ## CREATE_OUTPUT creates the desired output file
 
+  ## CREATE_OUTPUT creates the desired output file
   formatter = [];
   switch (options.format)
     case "html"
@@ -764,30 +776,32 @@ function ofile = create_output (doc, options)
       endfor
     endif
   endif
+
 endfunction
 
 
 function toc_cstr = get_toc (cstr, formatter)
+
   ## GET_TOC extracts the table of contents from a cellstring (e.g., doc.body)
   ## with each section headline as a cell in a returned cellstring.
-
   toc_cstr = cell ();
   for i = 1:numel (cstr)
     if (strcmp (cstr{i}.type, "section"))
       toc_cstr{end+1} = format_text (cstr{i}.content, formatter);
     endif
   endfor
+
 endfunction
 
 
 function str = format_output (cstr, formatter, options)
+
   ## FORMAT_OUTPUT steps through all blocks (doc.intro or doc.body) in cstr and
   ## produces a single result string with the source code in the desired output
   ## format.
   ##
   ##   formatter has the only knowledge how to enforce the target format
   ##   and produces for each block the necessary target format source string.
-
   str = "";
   for i = 1:numel (cstr)
     switch (cstr{i}.type)
@@ -814,12 +828,13 @@ endfunction
 
 
 function str = format_text (str, formatter)
+
   ## FORMAT_TEXT formats inline formats in strings.
   ##   These are: links, block/inline math, bold, italic, monospaced, (TM), (R)
 
   ## Helper to clarify the following regular expressions.  It is suitable for
   ## inline formatting, that is delimited literally at start and end by
-  ## `delim`.  `term` is an indicating character for the end delimiter.
+  ## 'delim'.  'term' is an indicating character for the end delimiter.
   ##
   ## Best explained by example ('^' start and '$' end of input):
   ##
@@ -844,9 +859,9 @@ function str = format_text (str, formatter)
 
   ## Regular expressions for the formats:
   ##
-  ## 1) Links "<http://www.someurl.com>"
-  ## 2) Links "<octave:Function TEXT>"
-  ## 3) Links "<http://www.someurl.com TEXT>"
+  ## 1) Links "<https://www.someurl.com>"
+  ## 2) Links "<octave:Function SOME TEXT>"
+  ## 3) Links "<https://www.someurl.com SOME TEXT>"
   ## 4) LaTeX block math "$$x^2$$"
   ## 5) LaTeX inline math "$x^2$"
   ## 6) Bold *text*
@@ -878,26 +893,27 @@ function str = format_text (str, formatter)
                        "once");
       switch (i)
         case 1
-          ## Links "<http://www.someurl.com>"
+          ## Links "<https://www.someurl.com>"
           url = cstr{j};
           cstr{j} = formatter ("link", url(2:end-1), url(2:end-1));
         case 2
-          ## Links "<octave:Function TEXT>"
-          idx = strfind (cstr{j}, " ");
+          ## Links "<octave:Function SOME TEXT>"
+          idx = strfind (cstr{j}, " ")(1);
           url = cstr{j};
           url = texinfo_esc (url(9:idx-1));
           v = version ();
-          if (v(end) == '+')
-            v = "interpreter";
+          if (v(end) != '0')  # No official release
+            v = "latest";
+          else
+            v = ["v" v];  # "v6.4.0" etc.
           endif
-          url = sprintf ( ...
-            "https://www.gnu.org/software/octave/doc/%s/XREF%s.html", v, url);
+          url = sprintf ("https://octave.org/doc/%s/XREF%s.html", v, url);
           txt = cstr{j};
           txt = format_text (txt(idx+1:end-1), formatter);
           cstr{j} = formatter ("link", url, txt);
         case 3
-          ## Links "<http://www.someurl.com TEXT>"
-          idx = strfind (cstr{j}, " ");
+          ## Links "<https://www.someurl.com SOME TEXT>"
+          idx = strfind (cstr{j}, " ")(1);
           url = cstr{j};
           url = url(2:idx-1);
           txt = cstr{j};
@@ -946,6 +962,7 @@ endfunction
 
 
 function doc = eval_code (doc, options)
+
   ## EVAL_CODE Third level parsing
   ##
   ##   Generates the output of the script code and takes care of generated
@@ -1049,6 +1066,7 @@ endfunction
 
 
 function cstr = eval_code_helper (__code__)
+
   ## EVAL_CODE_HELPER evaluates a given string with Octave code in an extra
   ## temporary context and returns a cellstring with the eval output.
 
@@ -1061,10 +1079,12 @@ function cstr = eval_code_helper (__code__)
   ## Split string by lines and preserve blank lines.
   cstr = strsplit (strrep (cstr, "\n\n", "\n \n"), "\n");
   eval_context ("save");
+
 endfunction
 
 
 function cstr = eval_context (op)
+
   ## EVAL_CONTEXT temporary evaluation context.
   persistent ctext
 
@@ -1082,7 +1102,7 @@ function cstr = eval_context (op)
       for i = 1:length (var_names)
         if (! any (strcmp (var_names{i}, forbidden_var_names)))
           ctext(var_names{i}) = evalin ("caller", var_names{i});
-        end
+        endif
       endfor
 
     case "load"
@@ -1101,13 +1121,14 @@ function cstr = eval_context (op)
       ## Do nothing
 
   endswitch
+
 endfunction
 
 
-## Note: Functional BIST tests are located in the `test/publish` directory.
+## Note: Functional BIST tests are located in the 'test/publish' directory.
 
 ## Test input validation
-%!error publish ()
+%!error <Invalid call> publish ()
 %!error publish (1)
 %!error <FILE does not exist> publish ("%%_non_existent_file_%%.m")
 %!error <only script files can be published> publish ("publish.m")
