@@ -1,4 +1,9 @@
-## Copyright (C) 2012-2019 John W. Eaton
+########################################################################
+##
+## Copyright (C) 2012-2022 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{val} =} recycle ()
@@ -31,16 +38,10 @@
 ## @seealso{delete, rmdir}
 ## @end deftypefn
 
-## Author: jwe
-
 function val = recycle (new_val)
 
   persistent current_state = "off";
 
-  if (nargin > 1)
-    print_usage ();
-  endif
-
   if (nargin == 0 || nargout > 0)
     val = current_state;
   endif
