@@ -1,5 +1,9 @@
-## Copyright (C) 2004-2019 Piotr Krzyzanowski
-## Copyright (C) 2016-2019 Cristiano Dorigo
+########################################################################
+##
+## Copyright (C) 2004-2022 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -16,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {@var{x} =} pcg (@var{A}, @var{b}, @var{tol}, @var{maxit}, @var{m1}, @var{m2}, @var{x0}, @dots{})
@@ -253,11 +259,6 @@
 ## @seealso{sparse, pcr, gmres, bicg, bicgstab, cgs}
 ## @end deftypefn
 
-## Author: Piotr Krzyzanowski <piotr.krzyzanowski@mimuw.edu.pl>
-## Modified by: Vittoria Rezzonico <vittoria.rezzonico@epfl.ch>
-##  - Add the ability to provide the pre-conditioner as two separate matrices
-
-
 function [x_min, flag, relres, iter_min, resvec, eigest] =...
          pcg (A, b, tol = [], maxit = [], M1 = [], M2 = [], x0 = [], varargin)
 
@@ -283,7 +284,7 @@ function [x_min, flag, relres, iter_min, resvec, eigest] =...
   b_norm = norm (b);
   if (b_norm == 0)
      if (n_arg_out < 2)
-       printf("The right hand side vector is all zero so pcg \n");
+       printf ("The right hand side vector is all zero so pcg \n");
        printf ("returned an all zero solution without iterating.\n");
      endif
      x_min = b;
@@ -292,7 +293,7 @@ function [x_min, flag, relres, iter_min, resvec, eigest] =...
      resvec = 0;
      iter_min = 0;
      eigest = [NaN, NaN];
-     return
+     return;
   endif
 
   x = x_pr = x_min = x0;
@@ -318,7 +319,7 @@ function [x_min, flag, relres, iter_min, resvec, eigest] =...
   while (resvec(iter-1,1) > tol * b_norm && iter < maxit)
     if (iter == 2) # Check whether M1 or M2 are singular
       try
-        warning ("error","Octave:singular-matrix","local")
+        warning ("error","Octave:singular-matrix","local");
         z = feval (M1fun, r, varargin{:});
         z = feval (M2fun, z, varargin{:});
       catch
@@ -356,7 +357,7 @@ function [x_min, flag, relres, iter_min, resvec, eigest] =...
     x += alpha * p;
     r -= alpha * w;
     resvec(iter, 1) = norm (r);
-    ## Chek if the iterated has minimum residual
+    ## Check if the iterated has minimum residual
     if (resvec (iter,1) <= resvec (iter_min + 1,1))
       x_min = x;
       iter_min = iter - 1;
@@ -394,7 +395,7 @@ function [x_min, flag, relres, iter_min, resvec, eigest] =...
       endif
     else
       eigest = [NaN, NaN];
-      warning ('pcg: eigenvalue estimate failed: matrix not positive definite?')
+      warning ('pcg: eigenvalue estimate failed: matrix not positive definite?');
     endif
     resvec(iter - 1, 2) = sqrt (r' * z);
     resvec  = resvec (1:(iter-1), :);
@@ -410,7 +411,7 @@ function [x_min, flag, relres, iter_min, resvec, eigest] =...
   elseif (resvec (1, 1) == 0)
     relres = 0;
   else
-    relres = resvec(iter_min+1, 1) ./ resvec(1, 1);
+    relres = resvec(iter_min+1, 1) ./ b_norm;
   endif
 
   iter -= 2; # compatibility
@@ -453,6 +454,7 @@ function [x_min, flag, relres, iter_min, resvec, eigest] =...
         printf ("has relative residual %d \n", relres);
     endswitch
   endif
+
 endfunction
 
 
@@ -671,3 +673,13 @@ endfunction
 %! [x, flag, relres] = pcg (A, b, [], 2, M);
 %! assert (norm (b - A * x) / norm (b), relres,  8 * eps);
 
+%!test <*59776>
+%! A = [ 1.00000000  -0.00054274  -0.00066848;
+%!      -0.00054274   1.00000000  -0.00060330;
+%!      -0.00066848  -0.00060330   1.00000000];
+%! b = [1 1 1]';
+%! [x, flag, relres, iter, resvec] = pcg (A, b, 1e-6, 4, [], [], [1; 1; 1]);
+%! assert (flag, 0);
+%! assert (relres, resvec(2) / norm (b));
+%! assert (iter, 1);
+
