@@ -1,4 +1,9 @@
-## Copyright (C) 2005-2019 Paul Kienzle
+########################################################################
+##
+## Copyright (C) 2005-2022 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,6 +20,8 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} test @var{name}
@@ -108,7 +115,7 @@
 ## any built-in demo blocks are extracted but not executed.  The text for all
 ## code blocks is concatenated and returned as @var{code} with @var{idx} being
 ## a vector of positions of the ends of each demo block.  For an easier way to
-## extract demo blocks from files, @xref{XREFexample,,example}.
+## extract demo blocks from files, @xref{XREFexample,,@code{example}}.
 ##
 ## If the second argument is @qcode{"explain"} then @var{name} is ignored and
 ## an explanation of the line markers used in @code{test} output reports is
@@ -118,7 +125,7 @@
 ## @end deftypefn
 
 ## Programming Note: All variables for test() must use the internal prefix "__".
-## Shared variables are eval'ed into the current workspace and therefore might
+## %!share variables are eval'ed into the current workspace and therefore might
 ## collide with the names used in the test.m function itself.
 
 function [__n, __nmax, __nxfail, __nbug, __nskip, __nrtskip, __nregression] = test (__name, __flag = "normal", __fid = [])
@@ -130,7 +137,13 @@ function [__n, __nmax, __nxfail, __nbug, __nskip, __nrtskip, __nregression] = te
   persistent __signal_file  = ">>>>> ";
   persistent __signal_skip  = "----- ";
 
-  if (nargin < 1 || nargin > 3)
+  __nxfail = 0;
+  __nbug = 0;
+  __nskip = 0;
+  __nrtskip = 0;
+  __nregression = 0;
+
+  if (nargin < 1)
     print_usage ();
   elseif (! isempty (__name) && ! ischar (__name))
     error ("test: NAME must be a string");
@@ -205,14 +218,15 @@ function [__n, __nmax, __nxfail, __nbug, __nskip, __nrtskip, __nregression] = te
     error ("test: unknown flag '%s'", __flag);
   endif
 
-  ## Locate the file to test.
+  ## Locate the file with tests.
   __file = file_in_loadpath (__name, "all");
-  if (isempty (__file))
-    __file = file_in_loadpath ([__name ".m"], "all");
-  endif
-  if (isempty (__file))
-    __file = file_in_loadpath ([__name ".cc"], "all");
-  endif
+  for suffix = {".m", ".cc", ".cc-tst", ".c-tst", ".C-tst", ".cpp-tst", ...
+                ".cxx-tst"}
+    if (! isempty (__file))
+      break;
+    endif
+    __file = file_in_loadpath ([__name, suffix{1}], "all");
+  endfor
   if (iscell (__file))
     if (isempty (__file))
       __file = "";
@@ -399,6 +413,15 @@ function [__n, __nmax, __nxfail, __nbug, __nskip, __nrtskip, __nregression] = te
           __vars = __vars(1:__idx(1)-1);
         endif
 
+        if (! isempty (deblank (__shared)))
+          ## Explicitly clear any existing shared variables so that
+          ## onCleanup actions will be executed.
+          __shared_vars = strtrim (ostrsplit (__shared, ","));
+          if (! isempty (__shared_vars))
+            clear (__shared_vars{:});
+          endif
+        endif
+
         ## Assign default values to variables.
         try
           __vars = deblank (__vars);
@@ -653,7 +676,7 @@ function [__n, __nmax, __nxfail, __nbug, __nskip, __nrtskip, __nregression] = te
       endif
 
       ## evaluate code for test, shared, and assert.
-      if (! isempty(__code))
+      if (! isempty (__code))
         try
           eval (sprintf ("function %s__test__(%s)\n%s\nendfunction",
                          __shared_r, __shared, __code));
@@ -710,7 +733,7 @@ function [__n, __nmax, __nxfail, __nbug, __nskip, __nrtskip, __nregression] = te
             && ! strcmp (__type, "xtest")
             && ! all (__shared == " "))
           fputs (__fid, "shared variables ");
-          eval (sprintf ("fdisp(__fid,var2struct(%s));", __shared));
+          eval (sprintf ("fdisp (__fid,var2struct(%s));", __shared));
         endif
         fflush (__fid);
       endif
@@ -740,9 +763,6 @@ function [__n, __nmax, __nxfail, __nbug, __nskip, __nrtskip, __nregression] = te
     end_unwind_protect
   endfor
 
-  ## Clear any functions created during test run.
-  eval (__clearfcn, "");
-
   ## Verify test file did not leak file descriptors.
   if (! isempty (setdiff (fopen ("all"), __fid_list_orig)))
     warning ("test: file %s leaked file descriptors\n", __file);
@@ -762,6 +782,16 @@ function [__n, __nmax, __nxfail, __nbug, __nskip, __nrtskip, __nregression] = te
              __file, sprintf (" %s", __leaked_vars{:}));
   endif
 
+  ## Explicitly clear any existing shared variables so that onCleanup
+  ## actions will be executed.
+  __shared_vars = strtrim (ostrsplit (__shared, ","));
+  if (! isempty (__shared_vars))
+    clear (__shared_vars{:});
+  endif
+
+  ## Clear any functions created during test run.
+  eval (__clearfcn, "");
+
   if (nargout == 0)
     if (__tests || __xfail || __xbug || __xskip || __xrtskip)
       if (__xfail || __xbug)
@@ -827,7 +857,7 @@ function pos = function_name (def)
   ## Find the end of the name.
   right = find (def == "(", 1);
   if (isempty (right))
-    return;
+    right = numel (def);
   endif
   right = find (def(1:right-1) != " ", 1, "last");
 
@@ -889,11 +919,13 @@ endfunction
 
 ## Strip '.*prefix:' from '.*prefix: msg\n' and strip trailing blanks.
 function msg = trimerr (msg, prefix)
+
   idx = index (msg, [prefix ":"]);
   if (idx > 0)
-    msg(1:idx+length(prefix)) = [];
+    msg(1:idx+length (prefix)) = [];
   endif
   msg = strtrim (msg);
+
 endfunction
 
 ## Strip leading blanks from string.
@@ -903,7 +935,14 @@ function str = trimleft (str)
 endfunction
 
 function body = __extract_test_code (nm)
-  fid = fopen (nm, "rt");
+
+  filedir = fileparts (nm);
+  if (is_same_file (filedir, pwd ()))
+    ## The canonical current directory is not added as key to the load path.
+    ## So it doesn't work as key for "dir_encoding". Use "." instead.
+    filedir = ".";
+  endif
+  fid = fopen (nm, "rt", "n", dir_encoding (filedir));
   body = "";
   if (fid >= 0)
     while (ischar (ln = fgets (fid)))
@@ -913,6 +952,7 @@ function body = __extract_test_code (nm)
     endwhile
     fclose (fid);
   endif
+
 endfunction
 
 
@@ -926,7 +966,7 @@ endfunction
 %!fail ("toeplitz ([1,2],[1,2;3,4])", msg2)
 %!fail ("toeplitz ([1,2;3,4],[1,2])", msg2)
 %!test fail ("toeplitz", "Invalid call to toeplitz")
-%!fail ("toeplitz (1, 2, 3)", "Invalid call to toeplitz")
+%!fail ("toeplitz (1, 2, 3)", "called with too many inputs")
 %!test assert (toeplitz ([1,2,3], [1,4]), [1,4; 2,1; 3,2])
 %!assert (toeplitz ([1,2,3], [1,4]), [1,4; 2,1; 3,2])
 %!demo toeplitz ([1,2,3,4],[1,5,6])
@@ -960,15 +1000,14 @@ endfunction
 
 ## Test 'fail' keyword
 %!fail ("test", "Invalid call to test")  # no args, generates usage()
-%!fail ("test (1,2,3,4)", "usage.*test") # too many args, generates usage()
+%!fail ("test (1,2,3,4)", "called with too many inputs") # too many args
 %!fail ('test ("test", "invalid")', "unknown flag")  # incorrect args
 %!fail ('garbage','garbage.*undefined')  # usage on nonexistent function should be
 
 ## Test 'error' keyword
-%!error test              # no args, generates usage()
-%!error test (1,2,3,4)    # too many args, generates usage()
+%!error <Invalid call> test              # no args, generates usage()
 %!error <unknown flag> test ("test", "invalid"); # incorrect args
-%!error test ("test", "invalid");  # test without pattern
+%!error test ("test", "invalid");        # test without pattern
 %!error <'garbage' undefined> garbage; # usage on nonexistent function is error
 
 ## Test 'warning' keyword
@@ -1040,7 +1079,7 @@ endfunction
 ## Test 'xtest' keyword
 %!xtest
 %! assert (1, 1);      # Test passes
-%!xtest <53613>
+%!test <53613>
 %! assert (0, 1);      # Test fails
 
 ## Test comment block.  It can contain anything.
@@ -1057,15 +1096,15 @@ endfunction
 ## All of the following tests should fail.  These tests should
 ## be disabled unless you are developing test() since users don't
 ## like to be presented with known failures.
-## %!test   error("---------Failure tests.  Use test('test','verbose',1)");
-## %!test   assert([a,b,c],[1,3,6]);   # variables have wrong values
+## %!test   error ("---------Failure tests.  Use test('test','verbose',1)");
+## %!test   assert ([a,b,c],[1,3,6]);   # variables have wrong values
 ## %!invalid                   # unknown block type
-## %!error  toeplitz([1,2,3]); # correct usage
+## %!error  toeplitz ([1,2,3]); # correct usage
 ## %!test   syntax errors)     # syntax errors fail properly
 ## %!shared garbage in         # variables must be comma separated
 ## %!error  syntax++error      # error test fails on syntax errors
 ## %!error  "succeeds.";       # error test fails if code succeeds
-## %!error <wrong pattern> error("message")  # error pattern must match
+## %!error <wrong pattern> error ("message")  # error pattern must match
 ## %!demo   with syntax error  # syntax errors in demo fail properly
 ## %!shared a,b,c
 ## %!demo                      # shared variables not available in demo
